package com.pointrmapwidget.util // replace this with the package you put this file

import android.content.Context
import com.pointrlabs.core.pathfinding.model.NodeType
import com.pointrlabs.core.pathfinding.model.PathDirection
import com.pointrlabs.core.pathfinding.model.PathDirectionType
import com.pointrsample.R // replace this with the R path of your project

class PathDirectionHelperMethods {

    /**
     * The following three methods [getDirectionMessageFromDirectionType] and [getMessageForPortal]
     * and [getNodeMessage] are used to generate a meaningful & translatable message from a PathDirection.
     * The R.string.xxx resources in these methods are given for demonstrative purposes, the implementor will need to
     * create relevant values in their project.
     * The project will not compile and give error if the code is copied directly without
     * adding the appropriate values to the string resources. 
     */
    // region message
    fun getDirectionMessageFromDirectionType(context: Context, direction: PathDirection, nextDirection: PathDirection? = null): String {
        val directionType = direction.directionType
        return when(directionType) {
            PathDirectionType.TurnLeft -> context.getString(R.string.turn_left)
            PathDirectionType.TurnRight -> context.getString(R.string.turn_right)
            PathDirectionType.TurnSlightlyLeft -> context.getString(R.string.turn_slightly_left)
            PathDirectionType.TurnSlightlyRight -> context.getString(R.string.turn_slightly_right)
            PathDirectionType.RotateBack -> context.getString(R.string.rotate_back)
            PathDirectionType.RotateLeft -> context.getString(R.string.rotate_left)
            PathDirectionType.RotateRight -> context.getString(R.string.rotate_right)
            PathDirectionType.UpwardPortal -> {
                getMessageForPortal(context, directionType, direction, nextDirection)
            }
            PathDirectionType.InterFacilityPortal -> {
                getMessageForPortal(context, directionType, direction, nextDirection)
            }
            PathDirectionType.DownwardPortal -> {
                getMessageForPortal(context, directionType, direction, nextDirection)
            }
            PathDirectionType.Destination -> {
                context.getString(R.string.destination)
            }
            else -> ""
        }
    }

    fun getMessageForPortal(
        context: Context,
        directionType: PathDirectionType,
        direction: PathDirection,
        nextDirection: PathDirection?
    ):String {
        val nextLevelTitleNullable = nextDirection?.location?.level?.getTitle()
        val nextLevelTitle = if (nextLevelTitleNullable.isNullOrEmpty()) {
            "level ${nextDirection?.location?.level?.getIndex()}"
        } else {
            nextLevelTitleNullable
        }

        val nextFacilityNullable = nextDirection?.location?.facility
        val nextFacilityTitle = if (nextFacilityNullable?.getInternalIdentifier() == 1) {
            "facility title of facility with id 1"
        } else if (nextFacilityNullable?.getInternalIdentifier() == 2) {
            "facility title of facility with id 2"
        } else {
            "facility title of facility with id 3"
        }

        val portalName = getNodeMessage(context, direction.nodeType)

        return when (directionType) {
            PathDirectionType.UpwardPortal -> context.getString(R.string.upward_portal, portalName, nextLevelTitle)
            PathDirectionType.DownwardPortal -> context.getString(R.string.downward_portal, portalName, nextLevelTitle)
            PathDirectionType.InterFacilityPortal -> context.getString(R.string.interfacility_portal, nextFacilityTitle)
            else -> ""
        }
    }

    fun getNodeMessage(context: Context, nodeType: NodeType): String {
        return when(nodeType) {
            NodeType.Ramp -> context.getString(R.string.ramp)
            NodeType.Escalator -> context.getString(R.string.escalator)
            NodeType.SkewedLift, NodeType.ServiceLift, NodeType.Lift -> context.getString(R.string.lift)
            NodeType.ServiceStairs, NodeType.RotationalStairs, NodeType.AngularStairs -> context.getString(R.string.stairs)
            NodeType.Metro -> context.getString(R.string.metro)
            NodeType.Shuttle -> context.getString(R.string.shuttle)
            NodeType.Train -> context.getString(R.string.train)
            NodeType.Steps -> context.getString(R.string.steps)
            else -> ""
        }
    }
    // endregion

    /**
     * The following method [getImageResOfDirection] method can be used to supply an image for
     * the direction depending on its PathDirectionType and NodeType. The R.drawable.xxx files
     * returned here are for demonstrative purposes, please replace them with actual drawables
     * in your project. Or else the project will not compile once you copy this code.
     */
    // region Image of direction
    fun getImageResOfDirection(
        directionType: PathDirectionType,
        nodeType: NodeType? = null
    ): Int {
        when (directionType) {
            PathDirectionType.Destination -> {
                return R.drawable.ic_destination_flag_blue
            }
            PathDirectionType.DownwardPortal -> {
                if (nodeType == NodeType.AngularStairs || nodeType == NodeType.RotationalStairs || nodeType == NodeType.ServiceStairs) {
                    return R.drawable.ic_stairs_blue
                } else if (nodeType == NodeType.Lift || nodeType == NodeType.ServiceLift || nodeType == NodeType.SkewedLift) {
                    return R.drawable.ic_elevator_blue
                } else if (nodeType == NodeType.Escalator) {
                    return R.drawable.ic_escalator_blue
                } else if (nodeType == NodeType.Ramp) {
                    return R.drawable.ic_ramp_blue
                } else {
                    return R.drawable.ic_escalator_blue
                }
            }
            PathDirectionType.RotateBack -> {
                return R.drawable.ic_arrow_rotate_back_blue
            }
            PathDirectionType.RotateLeft -> {
                return R.drawable.ic_arrow_rotate_left_blue
            }
            PathDirectionType.RotateRight -> {
                return R.drawable.ic_arrow_rotate_right_blue
            }
            PathDirectionType.TurnLeft -> {
                return R.drawable.ic_arrow_left_blue
            }
            PathDirectionType.TurnRight -> {
                return R.drawable.ic_arrow_right_blue
            }
            PathDirectionType.TurnSlightlyLeft -> {
                return R.drawable.ic_arrow_curved_left_blue
            }
            PathDirectionType.TurnSlightlyRight -> {
                return R.drawable.ic_arrow_curved_right_blue
            }
            PathDirectionType.UpwardPortal -> {
                if (nodeType == NodeType.AngularStairs || nodeType == NodeType.RotationalStairs || nodeType == NodeType.ServiceStairs) {
                    return R.drawable.ic_stairs_blue
                } else if (nodeType == NodeType.Lift || nodeType == NodeType.ServiceLift || nodeType == NodeType.SkewedLift) {
                    return R.drawable.ic_elevator_blue
                } else if (nodeType == NodeType.Escalator) {
                    return R.drawable.ic_escalator_blue
                } else if (nodeType == NodeType.Ramp) {
                    return R.drawable.ic_ramp_blue

                } else {
                    return R.drawable.ic_escalator_blue
                }
            }
            PathDirectionType.InterFacilityPortal -> {
                return R.drawable.ic_facility_change_blue

            }
            else -> {
                return R.drawable.ic_arrow_dashed_white
            }
        }
    }
    //endregion
}